import sys
import time
import random
import os 
import math
import pygame
from tools import *

class Terrain:
   def __init__(self,gamesize,terrain_img):
      self.gamesize = gamesize
      self.game_ratio = 1.0 #ratio of terrain real to pixel (i.e. 5m * game_ratio = 10px)
      self.background=terrain_img
      self.surface_pts=[]
      hght = self.background.get_height() 
      for i in range(self.background.get_width()):
         j=0
         while j < hght and self.background.get_at((i,j))[3] == 0: j += 1
         self.surface_pts.append(hght-j)
      self.screen_width=len(self.surface_pts)
      self.rpos=0 # real (float) position of left of screen
      self.spos=0 # screen position of left of screen
      self.loco_pos = 50 # distance from right side of screeen to lock the locomotive

   def initialize(self):
      self.spos=0
      self.rpos=0.0

   def incline(self,rx1,rx2): # Assume correct input
      dy_ave=0
      for i in range(int(rx1)+1,int(rx2)):
         dy_ave += self.get_y(i) - self.get_y(i-1)
      if rx2-rx1==0: return 0.0
      return math.atan((dy_ave/float(rx2-rx1)))

   def get_y(self,rx):
      x = int(rx*self.game_ratio)
      if x < 0: # Assume flat terrain outside of screen
         return self.surface_pts[0]/self.game_ratio
      if x >= len(self.surface_pts):
         return self.surface_pts[-1]/self.game_ratio
      return self.surface_pts[x]/self.game_ratio

   def get_screen_y(self,sx): # I am getting a screen x
      sx += self.spos # sx is coming in relative to the screen, so expand that 
      if sx < 0: # Assume flat terrain outside of screen
         return self.surface_pts[0]
      if sx >= len(self.surface_pts):
         return self.surface_pts[-1]
      return self.surface_pts[sx]

   def x_to_screen(self,rx): 
      return int(rx * self.game_ratio)-self.spos

   def render(self,screen,loco_x):
      sx = int(loco_x*self.game_ratio) # Where the locomotive is
      sr = sx+self.loco_pos
      self.spos = sr - screen.get_width()
      if self.spos < 0: self.spos = 0
      if self.spos+screen.get_width() > self.background.get_width():
         self.spos = self.background.get_width()-screen.get_width()
      self.rpos = self.spos/self.game_ratio
      bg_offset=self.gamesize[1]-self.background.get_height()
      screen.blit(self.background,(0,bg_offset),(self.spos,0,screen.get_width(),self.background.get_height()))


